/* *************************************************************************
                          csg.cpp  -  description
                             -------------------
    begin                : Wed Oct 16 2002       
    copyright            : (C) 2002 by Micha Riser
    email                : mriser@gmx.net
 ************************************************************************* */

/* *************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ************************************************************************* */

#include "csg.h"
#include "material3d.h"
#include "objectconstructionerror.h"

HitStatistics CSG::stat("CSG","Tests","Hits");

CSG::CSG(const CSG& c):Object3D(c), type(c.type), isclosed(true) {

    if (!c.isclosed) { throw ObjectConstructionError("Cannot copy open csg! Close it first"); }

/*  
    for(unsigned int i=0; i<c.objectlist.size(); i++) {
        objectlist.push_back(c.objectlist[i]->duplicate());
    }
*/


    listrefcount = c.listrefcount;
    (*listrefcount)++;
    objectlist = c.objectlist;

/*
    if (c.inverselist) {
        inverserefcount = c.inverserefcount;
        (*inverserefecount)++;
        inverselist = c.inverselist;
    }
*/

}

CSG::~CSG() {
    
    (*listrefcount)--;
    if (*listrefcount == 0) {
        delete(objectlist);
        delete(listrefcount);
    }
    
}

Object3D* CSG::duplicate() const {
    return new CSG(*this);
}

void CSG::addObject(Object3D* o) {
    if (isclosed || material) { throw ObjectConstructionError("Cannot add object to closed csg"); }
    objectlist->push_back(o);
}

void CSG::close() {
    if (isclosed) return;
    isclosed = true;
    calculateBoundingBox();
}

void CSG::inverse() {

    close();
    
    switch(type) {
        case UNION:     type = INTERSECT; break;
        case INTERSECT: type = MERGE; break;
        case MERGE:     type = INTERSECT; break;
    }

/*    if (inverselist) {
        vector<Object3D*> *tmp = objectlist;
        objectlist = inverselist;
        inverselist = objectlist;
  }*/
    
    if (*listrefcount != 1) {

        (*listrefcount)--;
        listrefcount = new int(1);
        vector<Object3D*> *tmp = objectlist;
        objectlist = new vector<Object3D*>();
        
        for(unsigned int i=0; i<tmp->size(); i++) {
            Object3D* dupl = (*tmp)[i]->duplicate();
            objectlist->push_back(dupl);
        }
        
    }
    
    for(unsigned int i=0; i<objectlist->size(); i++) {
        (*objectlist)[i]->inverse();
    }
        
    calculateBoundingBox();
    
}

void CSG::listInside(const Vector3& location, InsideList& l) const {

    if (type != UNION) {
        Object3D::listInside(location,l);
    } else {
        if (boundingbox && !boundingbox->isInsideAt(location) && !inversed) return;
        Vector3 local = location;
        transformPointIn(local);
        for(unsigned int i=0; i<objectlist->size(); i++) {
            (*objectlist)[i]->listInside(local,l);
        }
    }

}

Intersection* CSG::intersectUnBounded(const Ray &shoot) {

    stat.eval();

    CSGIntersection* i = 0;

    // fill Array with initial values
    for(unsigned int j = 0; j<objectlist->size(); j++) {

        Object3D* current = (*objectlist)[j];
        double t;

        if ( current->boundingbox &&
             (t=current->boundingbox->intersect(shoot))!=INTERSECTION_TIME_EPSILON) {
      
             if (t > INTERSECTION_TIME_EPSILON) {
                // boundingbox hit
                if (i == 0)  i = new CSGIntersection(this,shoot,INTERSECTION_INFINIT_TIME);
                i->listsize++;
                i->nodelist.push_back(Node(0,t,current));
            } else {
                // boundingbox missed
                if (type == INTERSECT) {
                    Vector3 local = Vector3::add( shoot.getOrigin(), shoot.getDirection() );
                    current->transformPointIn(local);
                    if (current->isInsideAt(local)==false) {delete(i); return 0;}
                } else if (type==MERGE) {
                    Vector3 local = Vector3::add( shoot.getOrigin(), shoot.getDirection() );
                    current->transformPointIn(local);
                    if (current->isInsideAt(local)==true) {delete(i); return 0;}
                }
            }
            continue;

        } else {

            // there is no boundingbox
            Ray localshoot = Ray(shoot);
            current->transformRayIn(localshoot);
            Intersection* ci = current->intersectUnBounded(localshoot);
            if (ci) {

                if (i == 0)  i = new CSGIntersection(this,shoot,INTERSECTION_INFINIT_TIME);
                DBL t = ci->currentTime();
                i->listsize++;
                i->nodelist.push_back(Node(ci,t,current));

            } else {
                if (type==INTERSECT) {
                    Vector3 local = Vector3::add( shoot.getOrigin(), shoot.getDirection() );
                    current->transformPointIn(local);
                    if (current->isInsideAt(local)==false) {delete(i); return 0;}
                } else if (type==MERGE) {
                    Vector3 local = Vector3::add( shoot.getOrigin(), shoot.getDirection() );
                    current->transformPointIn(local);
                    if (current->isInsideAt(local)==true) {delete(i); return 0;}
                }
            }
        }
    }

    if (i==0) return 0;

    /* build up structure */
    i->initListStructure();
    if (i->checkHead() == false) i->next();
    if (i->nodelist[1].time == INTERSECTION_INFINIT_TIME) {delete(i); return 0;}

    stat.success();
    return i;

}


Intersection* CSG::intersectUnBounded(const Ray &shoot, DBL tmax) {

    stat.eval();

    CSGIntersection* i = 0;

    // fill Array with initial values
    for(unsigned int j = 0; j<objectlist->size(); j++) {

        Object3D* current = (*objectlist)[j];
        double t;

        if ( current->boundingbox &&
             (t=current->boundingbox->intersect(shoot,tmax))!=INTERSECTION_TIME_EPSILON) {
            if (t > INTERSECTION_TIME_EPSILON) {
                // boundingbox hit            
                if (i == 0)  i = new CSGIntersection(this,shoot,tmax);
                i->listsize++;
                i->nodelist.push_back(Node(0,t,current));
            } else {
                // boundingbox missed
                if (type == INTERSECT) {
                    Vector3 local = Vector3::add( shoot.getOrigin(), shoot.getDirection() );
                    current->transformPointIn(local);
                    if (current->isInsideAt(local)==false) {delete(i); return 0;}
                } else if (type==MERGE) {
                    Vector3 local = Vector3::add( shoot.getOrigin(), shoot.getDirection() );
                    current->transformPointIn(local);
                    if (current->isInsideAt(local)==true) {delete(i); return 0;}
                }
            }
            continue;
              
        } else {

            // there is no boundingbox
            Ray localshoot = Ray(shoot);
            current->transformRayIn(localshoot);
            Intersection* ci = current->intersectUnBounded(localshoot,tmax);
            if (ci) {

                if (i == 0)  i = new CSGIntersection(this,shoot,tmax);
                DBL t = ci->currentTime();
                i->listsize++;
                i->nodelist.push_back(Node(ci,t,current));
                
            } else {
                if (type==INTERSECT) {
                    Vector3 local = Vector3::add( shoot.getOrigin(), shoot.getDirection() );
                    current->transformPointIn(local);
                    if (current->isInsideAt(local)==false) {delete(i); return 0;}
                } else if (type==MERGE) {
                    Vector3 local = Vector3::add( shoot.getOrigin(), shoot.getDirection() );
                    current->transformPointIn(local);
                    if (current->isInsideAt(local)==true) {delete(i); return 0;}
                }
            }
        }
    }

    if (i==0) return 0;

    /* build up structure */
    i->initListStructure();
    if (i->checkHead() == false) i->next();
    if (i->nodelist[1].time == INTERSECTION_INFINIT_TIME) {delete(i); return 0;}

    stat.success();
    
    return i;

}


bool CSG::isInsideAt(const Vector3 &locallocation) const {
  
  bool test;
  if (type==INTERSECT) test=false; else test=true;
  for(unsigned int i=0; i<objectlist->size(); i++) {
      if ((*objectlist)[i]->isInsideAtBounded(locallocation)==test) return test;
  }
  return !test;
}

void CSG::calculateBoundingBox() {
    
    Vector3 min, max;
    inversed = false;

    if (type==INTERSECT) {

        vector<Object3D*>::iterator it = objectlist->begin();

        min = Vector3::negativeInfinit();
        max = Vector3::positiveInfinit();

        for(; it!=objectlist->end(); ++it) {

            Object3D* current = *it;
            if (current->isInversed()==false) {

                if (current->boundingbox == 0) continue;
                BoundingBox checkbox = *current->boundingbox;

                // transform bb into CSG's space
                checkbox.transformate(*this);

                min.maximize( checkbox.getBoxMin() );
                max.minimize( checkbox.getBoxMax() );
            }
        }
        // check for infinit bounding and if so use union of boundingboxes as boundingbox
        // * add me * - done, see below

        // check for degenerated boundingbox
        // * add me *

    }

    if (
        type == UNION ||
        type == MERGE ||
        (type == INTERSECT && (min.equals(Vector3::negativeInfinit())) && (max.equals(Vector3::positiveInfinit())) )
       ) {

        vector<Object3D*>::iterator it = objectlist->begin();

        // UNION or MERGE

        max = Vector3::negativeInfinit();
        min = Vector3::positiveInfinit();

        for(; it!=objectlist->end(); ++it) {    

            Object3D* current = *it;
            if (current->boundingbox == 0) {
                delete(boundingbox);
                boundingbox = 0;
                return;
            }

            BoundingBox checkbox = *current->boundingbox;

            // transform bb into CSG's space
            checkbox.transformate(*this);

            min.minimize( checkbox.getBoxMin() );
            max.maximize( checkbox.getBoxMax() );

        }
       if (type==INTERSECT) inversed = true;
    }
  
    if ((!min.equals(Vector3::negativeInfinit()))&&(!max.equals(Vector3::positiveInfinit()))) {
        boundingbox = new BoundingBox(min,max);
//        cout << "BB: <" << min.a[0] <<","<<min.a[1]<<","<<min.a[2]<<"> - <"<<max.a[0]<<","<<max.a[1]<<","<<max.a[2]<<">\n";
    } else {
        delete(boundingbox);
        boundingbox = 0;
    }
     
}  


CSG::CSGIntersection::~CSGIntersection() {
    vector<Node>::iterator it = nodelist.begin();
    for(++it; it!=nodelist.end(); ++it) {
        delete(it->intersection);
    }
}

void CSG::CSGIntersection::next() {

    do{

        if (nodelist[1].intersection == 0) {
            
            // we had only the boundingbox intersection with this object so far
            Ray localshoot = ray;
            Object3D* obj = nodelist[1].object;
            obj->transformRayIn(localshoot);
            Intersection* ci = obj->intersectUnBounded(localshoot,tmax);
            if (ci) {       
                DBL t = ci->currentTime();
                nodelist[1].intersection = ci;
                nodelist[1].time = t;
            } else nodelist[1].time = INTERSECTION_INFINIT_TIME;

        } else {
            nodelist[1].intersection->next();
            nodelist[1].time = nodelist[1].intersection->currentTime();
        }

        // re-assure heap condition
        int j=1;
        int c=2;
        while(c<=listsize) {
            if (c!=listsize && nodelist[c+1].time<nodelist[c].time) c++;
            if (nodelist[j].time<=nodelist[c].time) break;
            Node tmp = nodelist[j];
            nodelist[j] = nodelist[c];
            nodelist[c] = tmp;
            j = c;
            c *= 2;
        }

        if (nodelist[1].time == INTERSECTION_INFINIT_TIME) return;

    } while (checkHead()==false);

    if (parent->type == UNION)  hitobject = nodelist[1].intersection->hitObject();
  
}

Vector3 CSG::CSGIntersection::normal() {
    
    Vector3 local = nodelist[1].intersection->normal();
    nodelist[1].object->transformNormalOut(local);
    return local;
    
}

Texture* CSG::CSGIntersection::texture() {

    Texture* ret = nodelist[1].intersection->texture();
    if (ret->colour == 0 || ret->finish == 0) {
        Vector3 p = ray.positionAtTime(nodelist[1].time);
        if (parent->material && parent->material->texture) {
            Texture* mytex = parent->material->texture->evaluateAt(p);
            if (ret->colour == 0) ret->setColour(mytex->colour);
            if (ret->finish == 0) ret->setFinish(mytex->finish);
            delete(mytex);      
        }
    }
    return ret;
  
}

bool CSG::CSGIntersection::checkHead() {

    if (nodelist[1].intersection == 0) return false;

    if (parent->type != UNION) {
        Vector3 ipoint = ray.positionAtTime(nodelist[1].time);
        for(int j=2; j<=listsize; j++) {
            if ( nodelist[j].object->isInsideAtBounded(ipoint) == (parent->type==MERGE) ) return false;
        }
    }

    return true;
    
}

void CSG::CSGIntersection::initListStructure() {

    for(int i=1; i<=listsize;i++) 	{
        int  k=i;
        while(k/2>0 &&  (nodelist[k/2].time>nodelist[k].time)) {
            Node tmp = nodelist[k/2];
            nodelist[k/2] = nodelist[k];
            nodelist[k] = tmp;
            k = k/2;
        }
    }
  
  if (parent->type == UNION) {
      if (nodelist[1].intersection) hitobject = nodelist[1].intersection->hitObject();    
  } else {
      hitobject = parent;
  }
  
}

void CSG::prepareToRender() {

    for(unsigned int i=0; i<objectlist->size(); i++) {
        (*objectlist)[i]->prepareToRender();
    }
                                                
}
