/* *************************************************************************
                          examples.cpp  -  description
                             -------------------
    begin                : Wed Oct 23 2002
    copyright            : (C) 2002 by Micha Riser
    email                : mriser@gmx.net
 ************************************************************************* */

/* *************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ************************************************************************* */

#include "examples.h"
 
#include "scene.h"
#include "statistics.h"
#include "cameraperspective.h"
#include "material3d.h"
#include "texture3d.h"
#include "interior.h"
#include "sphere.h"
#include "pointlight.h"
#include "csg.h"
#include "plane.h"
#include "isosurface.h"
#include "function.h"
#include "box.h"
#include "intervalmap.h"

//////////////////////////////////////////////////////////////////////////////
// Isosurface Scene
// Shows isosurface, reflection, refraction, patterned pigment, transparency
//////////////////////////////////////////////////////////////////////////////

class MyFunction: public Isosurface::IsosurfaceFunction {

public:

    MyFunction():evalcount("Function","Evaluations","") {}

    DBL evaluateAt(const Vector3& v) const {
        
        evalcount.eval();
        DBL x = v.a[0]; DBL y = v.a[1]; DBL z=v.a[2];
        //return v.length()-1+.2*( sin(10*v.a[0])+cos(9*v.a[1])+sin(8*v.a[2]) );
        //return -(v.a[0]+v.a[1]+v.a[2]);
        /*DBL f1 = sqrt(y*y + z*z) - 0.8;
        DBL f2 = fabs(x)+fabs(y)-1;
        DBL f3 = sqrt(y*y + z*z) - .6;
        DBL Blob_threshold = 0.05;
        return (1+Blob_threshold)-pow(Blob_threshold,f1)-pow(Blob_threshold,f2)*pow(Blob_threshold,f3);*/
        DBL k=5, a=0.95, b=0.8;
        return pow( (x*x+y*y+z*z-a*k*k),2 )-b*( pow(z-k,2)-2*x*x )*( pow(z+k,2)-2*y*y );

        //return pow(x,4)-5*x*x+pow(y,4)-5*y*y+pow(z,4)-5*z*z+11.8;
        //return v.length()-1;                                              

    }

private:
    mutable HitStatistics evalcount;

};


Pigment3D* setUpScene_Isosurface() {

    Scene* s = new Scene();

    Pigment3DMapped check = Pigment3DMapped(new Checker());
    check.insertCol(0,Colour3(1,0,0));
    check.insertCol(1,Colour3(0,0,0));

    Pigment3DMapped imap = Pigment3DMapped(new Gradient(Vector3(1,0,0)));
    imap.insertCol(0,Colour3(1,1,1));
    imap.insert(1,check.copy());

    Plane *pl = new Plane(Vector3(0,1,0));
    pl->setPigment(imap);
    pl->setFinish(Finish(0.1,0.6,0.2,1,1));
    pl->translate(Vector3(0,-2,0));
    s->addObject(pl);
    
    pl = new Plane(Vector3(0,0,1));
    pl->setPigment(Pigment3DConstant(Colour5(1,1,1,0,0)));
    pl->translate(Vector3(0,0,15));
    s->addObject(pl);

    MyFunction* myf = new MyFunction();
    Isosurface* myiso = new Isosurface(myf,new Sphere(6),160,true);
    myiso->scale(Vector3(.5,.5,.5));
    myiso->rotate(Vector3(0,35,0));
    myiso->translate(Vector3(0,0,5));
    myiso->setPigment(Pigment3DConstant(Colour5(1,1,1,0,0.65)));
    myiso->setInterior(Interior(1.33,0));
    myiso->setFinish(Finish(0.05,0.6,0.3,1,10));
    s->addObject(myiso);

    PointLight* p = new PointLight(Vector3(0,10,-8),Colour3(1),s);
    s->addLight(p);

    Material3D m = Material3D(new Texture3D(new Pigment3DConstant(Colour5(0)), new Finish(0.1,0.6,0,0*2,10)), new Interior() );
    s->setDefaultMaterial(m);

    s->object.prepareToRender();

    CameraPerspective* c = new CameraPerspective(s);

    return c;
};


/////////////////////////////////////////////////////
// Funciton to create 3d menger sponge
////////////////////////////////////////////////

Object3D* make_menger(double length, int depth) {

    if (depth>0) {

        Object3D* piece = make_menger(length/3,depth-1);

        CSG* u = new CSG(CSG::UNION);

        Object3D* q = piece->duplicate();
        q->translate(Vector3(-length/3,-length/3,-length/3));
        u->addObject(q);

        q = piece->duplicate();
        q->translate(Vector3(-length/3,-length/3,length/3));
        u->addObject(q);

        q = piece->duplicate();
        q->translate(Vector3(-length/3,length/3,-length/3));
        u->addObject(q);

        q = piece->duplicate();
        q->translate(Vector3(-length/3,length/3,length/3));
        u->addObject(q);

        q = piece->duplicate();
        q->translate(Vector3(length/3,-length/3,-length/3));
        u->addObject(q);

        q = piece->duplicate();
        q->translate(Vector3(length/3,-length/3,length/3));
        u->addObject(q);

        q = piece->duplicate();
        q->translate(Vector3(length/3,length/3,-length/3));
        u->addObject(q);

        q = piece->duplicate();
        q->translate(Vector3(length/3,length/3,length/3));
        u->addObject(q);


        //

        q = piece->duplicate();
        q->translate(Vector3(-length/3,-length/3,0));
        u->addObject(q);

        q = piece->duplicate();
        q->translate(Vector3(length/3,-length/3,0));
        u->addObject(q);

        q = piece->duplicate();
        q->translate(Vector3(-length/3,length/3,0));
        u->addObject(q);

        q = piece->duplicate();
        q->translate(Vector3(length/3,length/3,0));
        u->addObject(q);

        //

        q = piece->duplicate();
        q->translate(Vector3(0,-length/3,-length/3));
        u->addObject(q);

        q = piece->duplicate();
        q->translate(Vector3(0,length/3,-length/3));
        u->addObject(q);

        q = piece->duplicate();
        q->translate(Vector3(0,-length/3,length/3));
        u->addObject(q);

        q = piece->duplicate();
        q->translate(Vector3(0,length/3,length/3));
        u->addObject(q);

        //

        q = piece->duplicate();
        q->translate(Vector3(-length/3,0,-length/3));
        u->addObject(q);


        q = piece->duplicate();
        q->translate(Vector3(length/3,0,-length/3));
        u->addObject(q);

        q = piece->duplicate();
        q->translate(Vector3(-length/3,0,length/3));
        u->addObject(q);

        q = piece;
        q->translate(Vector3(length/3,0,length/3));
        u->addObject(q);

        u->close();

        return u;

    } else
    {
        //Object3D* o = new Sphere(length/2);
        Object3D* o = new Box(-length/2*1.0001,length/2*1.0001);
        return o;
    }

}


//////////////////////////////////////////////////////////////
// Cut menger sponge, recurse level 6
// Shows: CSG union, intersection, boundingboxes, reflections
//////////////////////////////////////////////////////////////

Pigment3D* setUpScene_Menger() {

    Scene* s = new Scene();

    CSG* csg = new CSG(CSG::INTERSECT);
    
    Object3D* menger = make_menger(3,6);
    csg->addObject(menger);

    Plane *pl = new Plane(Vector3(0,1,0));
    pl->translate(Vector3(0,1.1,0));
    pl->rotate(Vector3(-45,-40,0));
    csg->addObject(pl);

    csg->setPigment(Pigment3DConstant(Colour5(.7,.7,.7,0,0)));  
    csg->setFinish(Finish(0.03,0.3,0.6,1,5));
    csg->rotate(Vector3(11,23,0));
    csg->translate(Vector3(0,0,6));
    s->addObject(csg);
    
    PointLight* p = new PointLight(Vector3(0,10,-8),Colour3(1),s);
    s->addLight(p);

    PointLight* p2 = new PointLight(Vector3(-3,4,-6),Colour3(.5),s);
    s->addLight(p2);

    Material3D m = Material3D(new Texture3D(new Pigment3DConstant(Colour5(0)), new Finish(0.1,0.6,0,0,0)), new Interior() );
    s->setDefaultMaterial(m);

    s->object.prepareToRender();

    CameraPerspective* c = new CameraPerspective(s);

    return c;
  
};

//////////////////////////////////////////////////////////////
// Media (inversed) menger sponge
// shows: CSG, Scattering Media
//////////////////////////////////////////////////////////////

Pigment3D* setUpScene_MediaMenger() {

    Scene* s = new Scene();

    CSG* mengerc = new CSG(CSG::INTERSECT);
    
    Object3D* menger = make_menger(3,2);

    Box* b = new Box(Vector3(-1.499),Vector3(1.499));
    mengerc->addObject(b);
    
    menger->inverse();
    mengerc->addObject(menger);
    
    mengerc->setPigment(Pigment3DConstant(Colour5(1,1,1,0,1)));
    mengerc->rotate(Vector3(11,23,0));
    mengerc->translate(Vector3(0,0,6));
    Media3DSingle media(Colour3(0),Colour3(2.5),Colour3(2.5),Media3D::MIE_MURKY,s,0.0,0.0001,0.1);
    mengerc->setMedia(media);
    s->addObject(mengerc);

    Plane *pl = new Plane(Vector3(0,1,0));
    pl->setPigment(Pigment3DConstant(Colour5(0,0,.5,0,0)));    
    pl->translate(Vector3(0,-2.5,0));
    s->addObject(pl);

    Plane *pl2 = new Plane(Vector3(0,0,1));
    pl2->setPigment(Pigment3DConstant(Colour5(0,0,.5,0,0)));
    pl2->translate(Vector3(0,0,11.5));
    s->addObject(pl2);

    PointLight* p = new PointLight(Vector3(0,10,-8),Colour3(1*1.5),s);
    s->addLight(p);

    PointLight* p2 = new PointLight(Vector3(-3,4,-6),Colour3(.5*1.5),s);
    s->addLight(p2);

    Material3D m = Material3D(new Texture3D(new Pigment3DConstant(Colour5(0)), new Finish(0.1,0.6,0,0*2,10)), new Interior() );
    s->setDefaultMaterial(m);

    s->object.prepareToRender();

    CameraPerspective* c = new CameraPerspective(s);

    return c;

};


////////////////////////////////////////////////////////////////////////
// Media room
// Shows: CSG, scattering media
////////////////////////////////////////////////////////////////////////

Object3D* makeBox(Vector3 v1, Vector3 v2) {
    Box* b = new Box(v1,v2);
    b->setPigment(Pigment3DConstant(Colour5(.6,.6,.6,0,0)));
    return b;
}

Pigment3D* setUpScene_MediaRoom() {
  Scene* s = new Scene();

  CSG* diff = new CSG(CSG::INTERSECT);

  Object3D* b;
  b = new Box(Vector3(-10,-2,-10),Vector3(10,2,10));
  diff->addObject(b);
  b = new Box(Vector3(-9.9,-1.9,-9.9),Vector3(9.9,2.1,9.9));
  b->inverse();
  diff->addObject(b);
  diff->scale(Vector3(.4,.9,.9));
  diff->setPigment(Pigment3DConstant(Colour5(.6,.6,.6,0,0)));
  diff->setFinish(Finish(0.1,0.6,0,0,0));

  s->addObject(diff);

  s->addObject(makeBox(Vector3(-10,2,5),Vector3(10,2.1,7)));
  s->addObject(makeBox(Vector3(-10,2,8),Vector3(10,2.1,10)));
  s->addObject(makeBox(Vector3(-10,2,1),Vector3(10,2.1,4)));
  s->addObject(makeBox(Vector3(-10,2,-10),Vector3(10,2.1,-5)));
  s->addObject(makeBox(Vector3(-10,2,-4),Vector3(10,2.1,1)));
  s->addObject(makeBox(Vector3(-10,2,-10),Vector3(-8,2.1,10)));
  s->addObject(makeBox(Vector3(-6,2,-10),Vector3(-4,2.1,10)));
  s->addObject(makeBox(Vector3(-2,2,-11),Vector3(0,2.1,10)));

  s->addObject(makeBox(Vector3(2,2,-10),Vector3(4,2.1,10)));
  s->addObject(makeBox(Vector3(6,2,-10),Vector3(8,2.1,10)));

  Sphere* sp = new Sphere(.6,Vector3(1,1,7.5));
  sp->setPigment(Pigment3DConstant(Colour3(.3,0,0)));
  sp->setFinish(Finish(0.1,0.6,0.5,0,0));
  s->addObject(sp);

  b = new Box(Vector3(-12,-10,-10),Vector3(12,6,10));
  b->setPigment(Pigment3DConstant(Colour5(.5,.5,.5,0,1)));
  b->setFinish(Finish(0.1,1,0,0,0));
  Media3DSingle media(Colour3(0),Colour3(0.2),Colour3(0.2),Media3D::MIE_MURKY,s,0.05,0.0001,0.05);
  b->setMedia(media);
  s->addObject(b);

  PointLight* l = new PointLight(Vector3(10,20,0),Colour3(10),s);
  s->addLight(l);
  l = new PointLight(Vector3(-5,20,10),Colour3(10),s);
  s->addLight(l);

  Material3D m = Material3D(new Texture3D(new Pigment3DConstant(Colour5(0)), new Finish(0.1,0.6,0,0*2,10)), new Interior() );
  s->setDefaultMaterial(m);
  
  s->object.prepareToRender();

  CameraPerspective* c = new CameraPerspective(s);

  return c;

}

///////////////////////////////////////////////////////////////
// Recursive camera scene.
// Shows: pigments, camera-is-a-pigment principle
///////////////////////////////////////////////////////////////

Pigment3D* setUpScene_RecCamera() {

  Scene* s = new Scene();

  CameraPerspective* c = new CameraPerspective(s);
    
  Object3D* sp = new Box(Vector3(0),Vector3(1,1,0.01));
  sp->scale(Vector3(2));
  sp->rotate(Vector3(-1,-35,0));
  sp->translate(Vector3(-2,0,4));
  sp->setPigment(*c);
  sp->setFinish(Finish(1,0,0,0,0));
  s->addObject(sp);

  Pigment3DMapped check = Pigment3DMapped(new Checker());
  check.insertCol(0,Colour3(1,0,0));
  check.insertCol(1,Colour3(0,0,1));

  sp = new Box(Vector3(-.1,-.1,.01),Vector3(1.1,1.1,0.2));
  sp->scale(Vector3(2));  
  sp->setPigment(Pigment3DConstant(Colour5(0,1,0,0,0)));
  sp->rotate(Vector3(-1,-35,0));
  sp->translate(Vector3(-2,0,4));  
  s->addObject(sp);
  
  sp = new Plane(Vector3(0,1,0));
  sp->translate(Vector3(0,-2,0));
  sp->setPigment(check);
  s->addObject(sp);

  sp = new Plane(Vector3(0,0,1));
  sp->translate(Vector3(0,0,10));
  sp->setPigment(Pigment3DConstant(Colour3(1)));
  s->addObject(sp);

  PointLight* l = new PointLight(Vector3(0,0,0),Colour3(1),s);
  s->addLight(l);

  Material3D m = Material3D(new Texture3D(new Pigment3DConstant(Colour5(0)), new Finish(0.1,0.6,0,0,0)), new Interior() );
  s->setDefaultMaterial(m);


  return c;

}


///////////////////////////////////////////////////////////////
// Scene blending
/////////////////////////////////////////////////////////////

Pigment3D* setUpScene_BlendCamera() {

    Pigment3D* s1 = setUpScene_Isosurface();
    Pigment3D* s2 = setUpScene_Menger();

    Pigment3DMapped* imap = new Pigment3DMapped(new Gradient(Vector3(1,0,0)));
    imap->insert(0,s1);
    imap->insert(1,s2);

    return imap;

}
