/* *************************************************************************
                          lightray.cpp  -  description
                             -------------------
    begin                : Wed Oct 16 2002        
    copyright            : (C) 2002 by Micha Riser
    email                : mriser@gmx.net
 ************************************************************************* */

/* *************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ************************************************************************* */

#include "lightray.h"
#include "scene.h"

Colour3 LightRay::shoot(CLR& blockingerror) {

    Colour3 retcolour = colour;
    Colour3 absorption = Colour3(0);
    bool hasmedia = false;
    DBL lasttime = 0;
    CLR transmitamouth = 1;       
     
     Intersection* li = myscene->object.intersectUnBounded(*this,length);
     if (li != 0) {

         DBL time = li->currentTime();
         do {

             Media3D* media = inside.getMedia();
             if (media) {
                 media->sample(lasttime,time,*this,absorption);
                 hasmedia = true;
                 delete(media);
             }
  
             Texture* thisma = li->texture();
             transmitamouth *= thisma->colour->c[Colour5::TRANSMIT];
             delete(thisma);      
             if (transmitamouth == 0) { // *add* close to 0 can abort as well
                 delete(li);
                 blockingerror = colour.norm();
                 return Colour3(0);
             }
             inside.swap(li->hitObject());
             li->next();
             lasttime = time;
             time = li->currentTime();
         
         } while (time != INTERSECTION_INFINIT_TIME);

         delete(li);
     }

     // consider last interval from lasttime to
     // the lightsource (at time=length)
     Media3D* media = inside.getMedia();
     if (media) {
         media->sample(lasttime,length,*this,absorption);
         hasmedia = true;
     }

     if (hasmedia) {
         // Multiply colour with absorption
         absorption.negExp();
         retcolour.scale(absorption);
     }

     blockingerror = retcolour.norm();
     return retcolour.scaled(transmitamouth);

}
