/* *************************************************************************
                          main.cpp  -  description
                             -------------------
    begin                : Wed Oct 16 2002    
    copyright            : (C) 2002 by Micha Riser
    email                : mriser@gmx.net
 ************************************************************************* */

/* *************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ************************************************************************* */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream>

#include "statistics.h"
#include "render.h"
#include "ppmoutputstream.h"
#include "scene.h"
#include "objectconstructionerror.h"
#include "examples.h"

int width;
int height;

int parseArguments(int argc, char *argv[]) {

    // default values
    width = 640;
    height = 480;
    int scenenumber = 1;
    
    for(int i=0; i<argc; i++) {

        if (strcmp(argv[i],"-n")==0) {
            
            i++;
            scenenumber = atoi(argv[i]);

        } else if (strcmp(argv[i],"-w")==0) {

            i++;
            width = atoi(argv[i]);

        } else if (strcmp(argv[i],"-h")==0) {

            i++;
            height = atoi(argv[i]);

        }
        

    }

    return scenenumber;

}

int main(int argc, char *argv[])
{

    int scenenumber = parseArguments(argc,argv);

    Pigment3D* scene;
    Statistics::init();
    try{
        switch(scenenumber) {
            default:
            case 0:
                scene = setUpScene_Isosurface();
                break;
            case 1:
                scene = setUpScene_Menger();
                break;
            case 2:
                scene = setUpScene_MediaMenger();
                break;
            case 3:
                scene = setUpScene_MediaRoom();
                break;
            case 4:
                scene = setUpScene_RecCamera();
                break;

            case 5:
                scene = setUpScene_BlendCamera();
                break;

        }
    }
    catch(ObjectConstructionError e) {
        std::cout <<"setUpScene failed: "<<e.message<<std::endl;
        exit(1);
    }
    PixelOutputStream* pout = new PPMOutputStream("out.ppm",width,height);

    Render r( scene, pout, width, height );
    r.doRendering();

    delete(scene);

    Statistics::global->print();
    Statistics::finish();
        
}
