/* *************************************************************************
                          object3d.cpp  -  description
                             -------------------
    begin                : Wed Oct 16 2002    
    copyright            : (C) 2002 by Micha Riser
    email                : mriser@gmx.net
 ************************************************************************* */

/* *************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ************************************************************************* */

#include "object3d.h"
#include "material3d.h"
#include "intersection.h"
#include "rendererror.h"

Object3D::Object3D(const Object3D &o): Transformatable(o) {
    if (o.material) material = new Material3D(*o.material); else material = 0;
    if (o.boundingbox) boundingbox = new BoundingBox(*o.boundingbox); else boundingbox = 0;
    inversed = o.inversed;
}

Object3D::~Object3D() {
    delete(material);
    delete(boundingbox);
}

void Object3D::setMaterial(const Material3D &m) {
    if (!material) 
        material = new Material3D(m); 
    else 
        material->overlay(m);
}

void Object3D::setTexture(const Texture3D &t) {
    if (!material) material = new Material3D();
    if (!material->texture) 
        material->texture = new Texture3D(t); 
    else 
        material->texture->overlay(t);
}

void Object3D::setPigment(const Pigment3D &p) {
    if (!material) material = new Material3D();
    if (!material->texture) material->texture = new Texture3D(0,0);
    if (!material->texture->pigment) {
        material->texture->pigment = p.copy();
        material->texture->pigment->transformInverse(*this);
    }
}

void Object3D::setFinish(const Finish &f) {
    if (!material) material = new Material3D();
    if (!material->texture) material->texture = new Texture3D(0,0);
    if (!material->texture->finish) 
        material->texture->finish = new Finish(f);
}

void Object3D::setInterior(const Interior &i) {
    if (!material) material = new Material3D();
    if (!material->interior) 
        material->interior = new Interior(i); 
    else 
        material->interior->overlay(i);
}

void Object3D::setMedia(const Media3D &m) {
    if (!material) material = new Material3D();
    if (!material->interior) material->interior = new Interior();
    if (!material->interior->media) 
        material->interior->media = m.copy();
}

void Object3D::translate(const Vector3 &v) {
    Transformatable::translate(v);
    if (boundingbox) boundingbox->translate(v);
}

void Object3D::scale(Vector3 v) {
    Transformatable::scale(v);
    if (boundingbox) boundingbox->scale(v);
}

void Object3D::rotate(const Vector3 &v) {
    Matrix rot = Matrix::generateRotationMatrix(v);
    Transformatable::applyOrthogonalMatrix(rot);
    if (boundingbox) boundingbox->rotate(rot);
}

void Object3D::transform(const Transformatable& t) {
    throw RenderError("method Object3D::transform not implemented yet");
}

void Object3D::transformInverse(const Transformatable& t) {
    throw RenderError("method Object3D::transformInverse not implemented yet");
}


void Object3D::listInside(const Vector3& location, InsideList& l) const {
    if (material && material->interior && isInsideAtBounded(location)) l.add(this);
}
