/* *************************************************************************
                            scene.cpp  -  description
                             -------------------
    begin                : Wed Oct 16 2002    
    copyright            : (C) 2002 by Micha Riser
    email                : mriser@gmx.net
 ************************************************************************* */

/* *************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ************************************************************************* */

#include "scene.h"
#include "lightsource.h"

Scene::~Scene() {
    
    list<LightSource*>::iterator it = lights.begin();
    for(; it!=lights.end(); ++it) {
        delete(*it);
    }
    
}

Colour5 Scene::intersect(CameraRay& shoot) {
    finishParsing();
    return intersectRec(shoot);
}

Colour5 Scene::intersectRec(CameraRay& shoot) {

    if (shoot.shoot() == true) {

        Colour3 absorption = Colour3(0);
        Colour3 mediacontribution = shoot.evaluateMedia(absorption);
        absorption.exp();
        shoot.scaleWeight(absorption.minElement());

        Texture* hittexture = shoot.texture();

        Colour3 tcolour = Colour3(0,0,0);
        // do lighting
        list<LightSource*>::iterator it = lights.begin();
        for(; it!=lights.end(); ++it) {
            tcolour.add( (*it)->enlight(shoot, hittexture, shoot.getInside() ) );
        }
        tcolour.add(hittexture->enlightShadow(shoot));

        // if (tcolour.norm()*shoot.getWeight()*colourprecision >1) {return retcolour;}

        Colour3 retcolour = tcolour;
        CLR transmit = hittexture->colour->c[Colour5::TRANSMIT];
        if (transmit != 0) {
            CameraRay save = CameraRay(shoot);
            bool totalreflection = shoot.refract(transmit);

            if (totalreflection == false) {
                // do reflection
                if (hittexture->finish->reflection>0) {
                    save.reflect(hittexture->finish->reflection);
                    Colour5 rcol = intersectRec(save);
                    rcol.scale(hittexture->finish->reflection);
                    retcolour.add(rcol);
                }
                // do refraction        
                Colour5 tcol = intersectRec(shoot);
                tcol.scale(transmit);
                retcolour.add(tcol);
            } else {
                // do total reflection
                shoot.reflect(1);
                Colour5 rcol = intersectRec(shoot);
                retcolour.add(rcol);
            }
        } else {
            if (hittexture->finish->reflection>0) {
                // do reflection
                shoot.reflect(hittexture->finish->reflection);
                Colour5 rcol = intersectRec(shoot);
                rcol.scale(hittexture->finish->reflection);
                retcolour.add(rcol);
            }
        }

        delete(hittexture);

        retcolour.scale(absorption);
        return Colour3::add(retcolour,mediacontribution);

    }

    return bgcolour;

}


LightList* Scene::getLightList(const Vector3& location, CLR& maxerror) const {

    InsideList inside;
    object.listInside(location, inside);

    maxerror = 0;

    LightList* ret = 0;
    list<LightSource*>::const_iterator it = lights.begin();
    for(; it!=lights.end(); ++it) {

        LightRay lray = (*it)->shineTo(location, &inside);
        CLR thiserror;
        Colour3 c = lray.shoot(thiserror);
        if (thiserror>maxerror) maxerror = thiserror;
//        if (c.norm() != 0) {

            ret = new LightList(c, *it, ret);
//        }
        
    }
    return ret;

}

