/* *************************************************************************
                            texture.cpp  -  description
                             -------------------
    begin                : Wed Oct 16 2002    
    copyright            : (C) 2002 by Micha Riser
    email                : mriser@gmx.net
 ************************************************************************* */

/* *************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ************************************************************************* */

#include "texture.h"
#include "cameraray.h"

Colour3 Texture::enlightShadow(const CameraRay&) const {

    Colour5 ret = finish->ambient.scaled(*colour);
    ret.scale( 1-colour->c[Colour5::TRANSMIT] );
    return ret;

}

Colour3 Texture::enlightLight(LightRay &lray, const CameraRay& cray) const {

    Colour3 ret = Colour3(0);
    CLR null;
    Colour3 intensityorig = lray.shoot(null);

    // diffuse:
    DBL factor = Vector3::dotProd(cray.normal(), lray.getDirection());
    if (factor > 0) {
        Colour3 intensity = intensityorig.scaled(factor*finish->diffuse);
        intensity.scale(*colour);
        ret = intensity;
    }
    ret.scale(1-colour->c[Colour5::TRANSMIT]);
    
    // phong:
    if (factor>0 && finish->phong !=0) {
        DBL x = -2*Vector3::dotProd(cray.normal(),cray.getDirection());
        Vector3 reflecteddirection = cray.normal().scaled(x);
        reflecteddirection.add(cray.getDirection());
        DBL a = Vector3::dotProd(reflecteddirection, lray.getDirection());
        if (a>0) {
            a = pow(a,finish->phongsize);
            Colour3 c = intensityorig.scaled(a*finish->phong);
            ret.add(c);
        }
    }
    return ret;
  
}
