/* *************************************************************************
                            transformatable.cpp  -  description
                             -------------------
    begin                : Wed Oct 16 2002
    copyright            : (C) 2002 by Micha Riser
    email                : mriser@gmx.net
 ************************************************************************* */

/* *************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ************************************************************************* */

#include "transformatable.h"

void Transformatable::scale(Vector3 v) {

    if (transformation == 0) {

        inverse = new Matrix(Matrix::generateScaleMatrix(v));
        v.inverse();
        transformation = new Matrix(Matrix::generateScaleMatrix(v));
        translation.applyFromLeft(*inverse);
            
    } else {

        Matrix S = Matrix::generateScaleMatrix(v);
        inverse->applyFromLeft(S);
        translation.applyFromLeft(S);
        v.inverse();
        S = Matrix::generateScaleMatrix(v);
        transformation->applyFromLeft(S);
        
    }

}


void Transformatable::rotate(const Vector3 &v) {
    Matrix rot = Matrix::generateRotationMatrix(v);
    applyOrthogonalMatrix(rot);
}


void Transformatable::applyOrthogonalMatrix(const Matrix &m) {

    if (transformation) {

        inverse->applyFromLeft(m);
        translation.applyFromLeft(m);
        Matrix tmp = m;
        tmp.transpose();
        transformation->applyFromLeft(tmp);
            
    } else {
        
        transformation = new Matrix(m);
        transformation->transpose();        
        inverse = new Matrix(m);
        translation.applyFromLeft(m);

    }

}

void Transformatable::transform(const Transformatable& t) {
    // *check me for correctness*
    // unused
    
    translation.add(t.translation);
    if (t.transformation) {
        if (!transformation) {
            transformation = new Matrix(*t.transformation);
            inverse = new Matrix(*t.inverse);
        } else {
            transformation->applyFromLeft(*t.transformation);
            inverse->applyFromLeft(*t.inverse);
        }
    }
}

void Transformatable::transformInverse(const Transformatable& t) {
    // it is v = T(v'-t) where we have to solve for v':
    // v' = T^-1 (v + T*t)
    // Therefore: T' = T^-1, t' = -T*t

    Vector3 ts = t.translation;
    if (t.transformation) ts.applyFromLeft(*t.transformation);
    translation.sub(ts);

    if (t.transformation) 
    
    if (t.transformation) {
        if (!transformation) {

            transformation = new Matrix(*t.inverse);
            inverse = new Matrix(*t.transformation);

        } else {
        
            transformation->applyFromLeft(*t.inverse);
            Matrix tmp = *inverse;
            *inverse = *t.transformation;
            inverse->applyFromLeft(tmp);
            
        }
    }
}

void Transformatable::transformPointIn(Vector3 &p) const {
    // p' = T*(p-t)
    p.sub(translation);
    if (transformation) p.applyFromLeft(*transformation);
}

void Transformatable::transformNormalIn(Vector3 &p) const {
    //unused
    if (inverse) p.applyFromLeftTransposed(*inverse);
}

void Transformatable::transformNormalOut(Vector3 &p) const {
    if (transformation) p.applyFromLeftTransposed(*transformation);
}

void Transformatable::transformRayIn(Ray &r) const {
    r.origin.sub(translation);
    if (transformation) {
        r.origin.applyFromLeft(*transformation);
        r.direction.applyFromLeft(*transformation);
    }
}
